@@ -1,16 +1,10 @@
 use crate::internal::*;
 use crate::ser::*;
 use tract_core::ops::cast::Cast;
-use tract_core::ops::element_wise::ElementWiseOp;
 
 pub fn register(registry: &mut Registry) {
-    registry.register_element_wise(
-        "tract_core_cast",
-        TypeId::of::<tract_core::ops::cast::Cast>(),
-        cast_dump,
-        cast_parameters(),
-        cast_load,
-    );
+    registry.register_dumper(TypeId::of::<tract_core::ops::cast::Cast>(), cast_dump);
+    registry.register_primitive("tract_core_cast", &cast_parameters(), cast_load);
 }
 
 fn cast_parameters() -> Vec<Parameter> {
@@ -18,7 +12,7 @@ fn cast_parameters() -> Vec<Parameter> {
 }
 
 fn cast_dump(ast: &mut IntoAst, node: &TypedNode) -> TractResult<Option<Arc<RValue>>> {
-    let op = node.op_as::<ElementWiseOp>().unwrap().0.downcast_ref::<Cast>().unwrap();
+    let op = node.op_as::<Cast>().unwrap();
     let input = ast.mapping[&node.inputs[0]].clone();
     Ok(Some(invocation(
         "tract_core_cast",
@@ -42,5 +36,5 @@ fn cast_load(
     } else {
         invocation_dt.ok_or(format_err!("No datum type for cast"))?
     };
-    builder.wire(ElementWiseOp(Box::new(Cast { to })), &[input])
+    builder.wire(Cast { to }, &[input])
 }
